@@ -315,6 +315,14 @@ The file descriptors(fds) is a L<Sys::IO::PollfdArray> object.
 
 If the system call failed, an exception will be thrown with the error code set to the class id of the L<Error::System> class.
 
+=head2 getaddrinfo_raw
+
+  static method getaddrinfo_raw : int ($node : string, $service : string,
+                $hints : Sys::Socket::Addrinfo,
+                $res : Sys::Socket::Addrinfo[]);
+
+Same as the L</"getaddrinfo"> method, but doesn't throw exceptions related to system errors.
+
 =head2 getaddrinfo
 
   static method getaddrinfo : int ($node : string, $service : string,
@@ -333,6 +341,16 @@ The response must be defined. Otherwise an exception will be thrown.
 
 The length of the array of the response must be greater than or equal to 1. Otherwise an exception will be thrown.
 
+If a system error occur, an exception will be thrown with the error code set to the class id of the L<Error::System> class.
+
+=head2 getnameinfo_raw
+
+  static method getnameinfo_raw : int ($sa : Sys::Socket::Sockaddr, $salen : int,
+                $host : mutable string, $hostlen : int,
+                $serv : mutable string, $servlen : int, $flags : int);
+
+Same as the L</"getnameinfo"> method, but doesn't throw exceptions related to system errors.
+
 =head2 getnameinfo
 
   static method getnameinfo : int ($sa : Sys::Socket::Sockaddr, $salen : int,
@@ -345,6 +363,8 @@ See the detail of the L<getnameinfo|https://linux.die.net/man/3/getaddrinfo> fun
 
 The socket address(sa) is a L<Sys::Socket::Addrinfo|SPVM::Sys::Socket::Addrinfo> object.
 
+If a system error occur, an exception will be thrown with the error code set to the class id of the L<Error::System> class.
+
 =head2 gai_strerror
 
   static method gai_strerror : string($errcode : int);
@@ -353,22 +373,34 @@ The gai_strerror() function translates these error codes to a human readable str
 
 See the detail of the L<getnameinfo|https://linux.die.net/man/3/gai_strerror> function in the case of Linux.
 
-=head2 ioctl_int_portable
+=head2 ioctl_int
 
-  static method ioctl_int_portable : int ($fd : int, $request : int, $request_arg_ref : int*);
+  static method ioctl_int : int ($fd : int, $request : int, $request_arg_ref : int*);
 
 The same as L<ioctl_int in Sys::IO|SPVM::Sys::IO/"ioctl_int">, but portable in socket.
 
-=head2 poll_portable
+=head2 poll
 
-  static method poll_portable : int ($fds : Sys::IO::PollfdArray, $nfds : int, $timeout : int);
+  static method poll : int ($fds : Sys::IO::PollfdArray, $nfds : int, $timeout : int);
 
 The same as L<poll in Sys::IO|SPVM::Sys::IO/"poll">, but portable in socket.
 
 The file descriptors(fds) is a L<Sys::IO::PollfdArray> object.
 
-=head2 close_portable
+=head2 close
 
-  static method close_portable : int ($fd : int);
+  static method close : int ($fd : int);
 
 The same as L<close in Sys::IO|SPVM::Sys::IO/"close">, but portable in socket.
+
+=head2 socket_errno
+
+  static method socket_errno : int ();
+
+Portalbe C<errno> related to the errors of the socket.
+
+=head2 socket_strerror
+
+  static method socket_strerror : string ($errno : int, $length : int);
+
+Portalbe C<strerror> related to the errors of the socket.
